/*                                                                            */
/* CDDL HEADER START                                                          */
/*                                                                            */
/* The contents of this file are subject to the terms of the Common           */
/* Development and Distribution License Version 1.0 (the "License").          */
/*                                                                            */
/* You can obtain a copy of the license at                                    */
/* http://www.opensource.org/licenses/CDDL-1.0.  See the License for the      */
/* specific language governing permissions and limitations under the License. */
/*                                                                            */
/* When distributing Covered Code, include this CDDL HEADER in each file and  */
/* include the License file in a prominent location with the name             */
/* LICENSE.CDDL.                                                              */
/* If applicable, add the following below this CDDL HEADER, with the fields   */
/* enclosed by brackets "[]" replaced with your own identifying information:  */
/*                                                                            */
/* Portions Copyright (c) [yyyy] [name of copyright owner].                   */
/* All rights reserved.                                                       */
/*                                                                            */
/* CDDL HEADER END                                                            */
/*                                                                            */

/*                                                                            */
/* Copyright (c) 2016--2019, Regents of the University of Minnesota.          */
/* All rights reserved.                                                       */
/*                                                                            */
/* Contributors:                                                              */
/*    Ryan S. Elliott                                                         */
/*                                                                            */

/*                                                                            */
/* Release: This file is part of the kim-api.git repository.                  */
/*                                                                            */


#ifndef KIM_SPECIES_NAME_INC_
#define KIM_SPECIES_NAME_INC_

#define ID_electron 0
#define ID_H 1
#define ID_He 2
#define ID_Li 3
#define ID_Be 4
#define ID_B 5
#define ID_C 6
#define ID_N 7
#define ID_O 8
#define ID_F 9
#define ID_Ne 10
#define ID_Na 11
#define ID_Mg 12
#define ID_Al 13
#define ID_Si 14
#define ID_P 15
#define ID_S 16
#define ID_Cl 17
#define ID_Ar 18
#define ID_K 19
#define ID_Ca 20
#define ID_Sc 21
#define ID_Ti 22
#define ID_V 23
#define ID_Cr 24
#define ID_Mn 25
#define ID_Fe 26
#define ID_Co 27
#define ID_Ni 28
#define ID_Cu 29
#define ID_Zn 30
#define ID_Ga 31
#define ID_Ge 32
#define ID_As 33
#define ID_Se 34
#define ID_Br 35
#define ID_Kr 36
#define ID_Rb 37
#define ID_Sr 38
#define ID_Y 39
#define ID_Zr 40
#define ID_Nb 41
#define ID_Mo 42
#define ID_Tc 43
#define ID_Ru 44
#define ID_Rh 45
#define ID_Pd 46
#define ID_Ag 47
#define ID_Cd 48
#define ID_In 49
#define ID_Sn 50
#define ID_Sb 51
#define ID_Te 52
#define ID_I 53
#define ID_Xe 54
#define ID_Cs 55
#define ID_Ba 56
#define ID_La 57
#define ID_Ce 58
#define ID_Pr 59
#define ID_Nd 60
#define ID_Pm 61
#define ID_Sm 62
#define ID_Eu 63
#define ID_Gd 64
#define ID_Tb 65
#define ID_Dy 66
#define ID_Ho 67
#define ID_Er 68
#define ID_Tm 69
#define ID_Yb 70
#define ID_Lu 71
#define ID_Hf 72
#define ID_Ta 73
#define ID_W 74
#define ID_Re 75
#define ID_Os 76
#define ID_Ir 77
#define ID_Pt 78
#define ID_Au 79
#define ID_Hg 80
#define ID_Tl 81
#define ID_Pb 82
#define ID_Bi 83
#define ID_Po 84
#define ID_At 85
#define ID_Rn 86
#define ID_Fr 87
#define ID_Ra 88
#define ID_Ac 89
#define ID_Th 90
#define ID_Pa 91
#define ID_U 92
#define ID_Np 93
#define ID_Pu 94
#define ID_Am 95
#define ID_Cm 96
#define ID_Bk 97
#define ID_Cf 98
#define ID_Es 99
#define ID_Fm 100
#define ID_Md 101
#define ID_No 102
#define ID_Lr 103
#define ID_Rf 104
#define ID_Db 105
#define ID_Sg 106
#define ID_Bh 107
#define ID_Hs 108
#define ID_Mt 109
#define ID_Ds 110
#define ID_Rg 111
#define ID_Cn 112
#define ID_Nh 113
#define ID_Fl 114
#define ID_Mc 115
#define ID_Lv 116
#define ID_Ts 117
#define ID_Og 118
#define ID_user01 201
#define ID_user02 202
#define ID_user03 203
#define ID_user04 204
#define ID_user05 205
#define ID_user06 206
#define ID_user07 207
#define ID_user08 208
#define ID_user09 209
#define ID_user10 210
#define ID_user11 211
#define ID_user12 212
#define ID_user13 213
#define ID_user14 214
#define ID_user15 215
#define ID_user16 216
#define ID_user17 217
#define ID_user18 218
#define ID_user19 219
#define ID_user20 220

#endif /* KIM_SPECIES_NAME_INC_ */
